@@ -31,6 +31,14 @@
  *
  * By using a #GtkClutterEmbed widget is possible to build, show and
  * interact with a scene built using Clutter inside a GTK+ application.
+ *
+ * <warning><para>Though #GtkClutterEmbed is a #GtkContainer subclass,
+ * it is not a real GTK+ container; #GtkClutterEmbed is required to
+ * implement #GtkContainer in order to embed a #GtkWidget through the
+ * #GtkClutterActor class. Calling gtk_container_add() on a #GtkClutterEmbed
+ * will trigger an assertion. It is strongly advised not to override the
+ * #GtkContainer implementation when subclassing
+ * #GtkClutterEmbed.</para></warning>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -173,7 +181,7 @@ pick_embedded_child (GdkWindow       *offscreen_window,
   GtkWidget *widget;
 
   a = clutter_stage_get_actor_at_pos (CLUTTER_STAGE (priv->stage),
-				      CLUTTER_PICK_ALL,
+				      CLUTTER_PICK_REACTIVE,
 				      widget_x, widget_y);
   if (GTK_CLUTTER_IS_ACTOR (a))
     {
@@ -269,6 +277,7 @@ gtk_clutter_embed_realize (GtkWidget *widget)
    */
   attributes.event_mask = gtk_widget_get_events (widget)
                         | GDK_EXPOSURE_MASK
+                        | GDK_SCROLL_MASK
                         | GDK_BUTTON_PRESS_MASK
                         | GDK_BUTTON_RELEASE_MASK
                         | GDK_KEY_PRESS_MASK
@@ -612,7 +621,14 @@ gtk_clutter_embed_add (GtkContainer *container,
 {
   GtkClutterEmbedPrivate *priv = GTK_CLUTTER_EMBED (container)->priv;
 
-  g_assert (GTK_CLUTTER_IS_OFFSCREEN (widget));
+#ifndef G_DISABLE_ASSERT
+  if (G_UNLIKELY (!GTK_CLUTTER_IS_OFFSCREEN (widget)))
+    {
+      g_critical ("Widgets of type '%s' do not support children.",
+                  G_OBJECT_TYPE_NAME (container));
+      return;
+    }
+#endif
 
   priv->children = g_list_prepend (priv->children, widget);
   gtk_widget_set_parent (widget, GTK_WIDGET (container));
@@ -672,11 +688,12 @@ static void
 gtk_clutter_embed_state_flags_changed (GtkWidget *widget,
                                        GtkStateFlags prev_state_flags)
 {
-  GTK_WIDGET_CLASS (gtk_clutter_embed_parent_class)->state_flags_changed 
-    (widget, prev_state_flags);
+  if (GTK_WIDGET_CLASS (gtk_clutter_embed_parent_class)->state_flags_changed != NULL)
+    GTK_WIDGET_CLASS (gtk_clutter_embed_parent_class)->state_flags_changed (widget, prev_state_flags);
 
   gtk_container_forall (GTK_CONTAINER (widget),
-                        (GtkCallback) gtk_widget_queue_draw, NULL);
+                        (GtkCallback) gtk_widget_queue_draw,
+                        NULL);
 }
 
 static void
